@@ -26,9 +26,12 @@ G_BEGIN_DECLS
 
 typedef enum {
   GCLUE_TOWER_TEC_UNKNOWN = 0,
-  GCLUE_TOWER_TEC_3G = 1,
-  GCLUE_TOWER_TEC_4G = 2,
+  GCLUE_TOWER_TEC_2G = 1,
+  GCLUE_TOWER_TEC_3G = 2,
+  GCLUE_TOWER_TEC_4G = 3,
+  GCLUE_TOWER_TEC_NO_FIX = 99,
 } GClueTowerTec;
+# define GCLUE_TOWER_TEC_MAX_VALID GCLUE_TOWER_TEC_4G
 
 typedef struct _GClue3GTower GClue3GTower;
 
