@@ -21,7 +21,9 @@
  */
 
 #include <glib/gi18n.h>
+#include <glib-unix.h>
 #include <config.h>
+#include <signal.h>
 
 #include "gclue-service-manager.h"
 #include "gclue-service-client.h"
@@ -31,37 +33,33 @@
 #include "gclue-locator.h"
 #include "gclue-config.h"
 
-/* 20 seconds as milliseconds */
-#define AGENT_WAIT_TIMEOUT 20000
-/*  20 seconds as microseconds */
-#define AGENT_WAIT_TIMEOUT_USEC (20 * G_USEC_PER_SEC)
-
 static void
 gclue_service_manager_manager_iface_init (GClueDBusManagerIface *iface);
 static void
 gclue_service_manager_initable_iface_init (GInitableIface *iface);
 
+struct _GClueServiceManagerPrivate
+{
+        GDBusConnection *connection;
+        GList *clients;
+        GHashTable *agents;
+        GQueue *clients_waiting_agent;
+
+        guint last_client_id;
+        guint num_clients;
+        guint unix_signal_source;
+
+        GClueLocator *locator;
+};
+
 G_DEFINE_TYPE_WITH_CODE (GClueServiceManager,
                          gclue_service_manager,
                          GCLUE_DBUS_TYPE_MANAGER_SKELETON,
                          G_IMPLEMENT_INTERFACE (GCLUE_DBUS_TYPE_MANAGER,
                                                 gclue_service_manager_manager_iface_init)
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
-                                                gclue_service_manager_initable_iface_init))
-
-struct _GClueServiceManagerPrivate
-{
-        GDBusConnection *connection;
-        GHashTable *clients;
-        GHashTable *agents;
-
-        guint num_clients;
-        gint64 init_time;
-
-        GClueLocator *locator;
-
-        gboolean active;
-};
+                                                gclue_service_manager_initable_iface_init)
+                         G_ADD_PRIVATE (GClueServiceManager))
 
 enum
 {
@@ -73,15 +71,60 @@ enum
 
 static GParamSpec *gParamSpecs[LAST_PROP];
 
+typedef struct
+{
+        GClueDBusManager *manager;
+        GDBusMethodInvocation *invocation;
+        GClueClientInfo *client_info;
+        gboolean reuse_client;
+
+} OnClientInfoNewReadyData;
+
+static gboolean
+log_client_list (gpointer user_data)
+{
+        GClueServiceManager *manager = GCLUE_SERVICE_MANAGER (user_data);
+        GList *l;
+
+        g_message ("SIGUSR1 received, printing client list:");
+        if (!manager->priv->clients) {
+                g_message ("    (No clients)");
+                return G_SOURCE_CONTINUE;
+        }
+        g_message ("    System  Active  UID     Id");
+        for (l = manager->priv->clients; l != NULL; l = l->next) {
+                GClueDBusClient *dbus_client = GCLUE_DBUS_CLIENT (l->data);
+                GClueClientInfo *client_info =
+                        gclue_service_client_get_client_info (GCLUE_SERVICE_CLIENT (l->data));
+                GClueConfig *config;
+                guint32 uid;
+                const char *system, *active, *id;
+                const char *y = "x";
+                const char *n = "-";
+                id = gclue_dbus_client_get_desktop_id (dbus_client);
+                uid = gclue_client_info_get_user_id (client_info);
+                config = gclue_config_get_singleton ();
+                system = gclue_config_is_system_component (config, id) ? y : n;
+                active = gclue_dbus_client_get_active (dbus_client) ? y : n;
+                g_message ("    %s       %s       %-7d %s", system, active, uid, id);
+        }
+        return G_SOURCE_CONTINUE;
+}
+
+static void
+on_client_info_new_ready_data_free (gpointer data)
+{
+        g_slice_free (OnClientInfoNewReadyData, data);
+}
+
 static void
 sync_in_use_property (GClueServiceManager *manager)
 {
-        gboolean in_use = FALSE, active = FALSE;
-        GList *clients, *l;
+        gboolean in_use = FALSE;
+        GList *l;
         GClueDBusManager *gdbus_manager;
 
-        clients = g_hash_table_get_values (manager->priv->clients);
-        for (l = clients; l != NULL; l = l->next) {
+        for (l = manager->priv->clients; l != NULL; l = l->next) {
                 GClueDBusClient *client = GCLUE_DBUS_CLIENT (l->data);
                 GClueConfig *config;
                 const char *id;
@@ -89,7 +132,6 @@ sync_in_use_property (GClueServiceManager *manager)
                 id = gclue_dbus_client_get_desktop_id (client);
                 config = gclue_config_get_singleton ();
 
-                active |= gclue_dbus_client_get_active (client);
                 if (gclue_dbus_client_get_active (client) &&
                     !gclue_config_is_system_component (config, id)) {
                         in_use = TRUE;
@@ -97,37 +139,74 @@ sync_in_use_property (GClueServiceManager *manager)
                         break;
                 }
         }
-        g_list_free (clients);
 
-        if (manager->priv->active != active) {
-                manager->priv->active = active;
-                g_object_notify (G_OBJECT (manager), "active");
-        }
         gdbus_manager = GCLUE_DBUS_MANAGER (manager);
         if (in_use != gclue_dbus_manager_get_in_use (gdbus_manager))
                 gclue_dbus_manager_set_in_use (gdbus_manager, in_use);
 }
 
+static int
+compare_client_bus_name (gconstpointer a,
+                         gconstpointer b)
+{
+        GClueClientInfo *info;
+        const char *bus_name = (const char *) b;
+
+        info = gclue_service_client_get_client_info (GCLUE_SERVICE_CLIENT (a));
+
+        return g_strcmp0 (bus_name, gclue_client_info_get_bus_name (info));
+}
+
+static void
+delete_client (GClueServiceManager *manager,
+                GCompareFunc        compare_func,
+                gpointer            compare_func_data)
+{
+        GClueServiceManagerPrivate *priv = manager->priv;
+        GList *l;
+
+        l = priv->clients;
+        while (l != NULL) {
+                GList *next = l->next;
+
+                if (compare_func (l->data, compare_func_data) == 0) {
+                        g_object_unref (G_OBJECT (l->data));
+                        priv->clients = g_list_delete_link (priv->clients, l);
+                        priv->num_clients--;
+                        if (priv->num_clients == 0) {
+                                g_object_notify (G_OBJECT (manager), "active");
+                        }
+                }
+                l = next;
+        }
+
+        g_debug ("Number of connected clients: %u", priv->num_clients);
+        sync_in_use_property (manager);
+}
+
+static void
+on_client_notify_active (GObject    *gobject,
+                         GParamSpec *pspec,
+                         gpointer    user_data)
+{
+        sync_in_use_property (GCLUE_SERVICE_MANAGER (user_data));
+}
+
 static void
 on_peer_vanished (GClueClientInfo *info,
                   gpointer         user_data)
 {
-        GClueServiceManager *manager = GCLUE_SERVICE_MANAGER (user_data);
+        const char *bus_name;
 
-        g_hash_table_remove (manager->priv->clients,
-                             gclue_client_info_get_bus_name (info));
-        g_debug ("Number of connected clients: %u",
-                 g_hash_table_size (manager->priv->clients));
-        sync_in_use_property (manager);
+        bus_name = gclue_client_info_get_bus_name (info);
+        g_debug ("Client `%s` vanished. Dropping associated client objects",
+                 bus_name);
+
+        delete_client (GCLUE_SERVICE_MANAGER (user_data),
+                       compare_client_bus_name,
+                       (char *) bus_name);
 }
 
-typedef struct
-{
-        GClueDBusManager *manager;
-        GDBusMethodInvocation *invocation;
-        GClueClientInfo *client_info;
-} OnClientInfoNewReadyData;
-
 static gboolean
 complete_get_client (OnClientInfoNewReadyData *data)
 {
@@ -135,16 +214,38 @@ complete_get_client (OnClientInfoNewReadyData *data)
         GClueServiceClient *client;
         GClueClientInfo *info = data->client_info;
         GClueAgent *agent_proxy = NULL;
-        GError *error = NULL;
-        char *path;
+        g_autoptr(GError) error = NULL;
+        g_autofree char *path = NULL;
         guint32 user_id;
 
+        /* Disconnect on_peer_vanished_before_completion, if it's there */
+        g_signal_handlers_disconnect_by_data (info, data);
+
         user_id = gclue_client_info_get_user_id (info);
         agent_proxy = g_hash_table_lookup (priv->agents,
                                            GINT_TO_POINTER (user_id));
 
+        if (data->reuse_client) {
+                GList *node;
+                const char *peer;
+
+                peer = g_dbus_method_invocation_get_sender (data->invocation);
+                node = g_list_find_custom (priv->clients,
+                                           peer,
+                                           compare_client_bus_name);
+                if (node != NULL) {
+                        GClueServiceClient *client;
+
+                        client = GCLUE_SERVICE_CLIENT (node->data);
+                        path = g_strdup
+                                (gclue_service_client_get_path (client));
+
+                        goto client_created;
+                }
+        }
+
         path = g_strdup_printf ("/org/freedesktop/GeoClue2/Client/%u",
-                                ++priv->num_clients);
+                                ++priv->last_client_id);
 
         client = gclue_service_client_new (info,
                                            path,
@@ -154,24 +255,32 @@ complete_get_client (OnClientInfoNewReadyData *data)
         if (client == NULL)
                 goto error_out;
 
-        g_hash_table_insert (priv->clients,
-                             g_strdup (gclue_client_info_get_bus_name (info)),
-                             client);
-        g_debug ("Number of connected clients: %u",
-                 g_hash_table_size (priv->clients));
+        priv->clients = g_list_prepend (priv->clients, client);
+        priv->num_clients++;
+        if (priv->num_clients == 1) {
+                g_object_notify (G_OBJECT (data->manager), "active");
+        }
+        g_debug ("Number of connected clients: %u", priv->num_clients);
 
-        g_signal_connect (info,
-                          "peer-vanished",
-                          G_CALLBACK (on_peer_vanished),
-                          data->manager);
-        g_signal_connect_swapped (client,
-                                  "notify::active",
-                                  G_CALLBACK (sync_in_use_property),
-                                  data->manager);
+        g_signal_connect_object (client,
+                                 "notify::active",
+                                 G_CALLBACK (on_client_notify_active),
+                                 data->manager, 0);
 
-        gclue_dbus_manager_complete_get_client (data->manager,
-                                                data->invocation,
-                                                path);
+        g_signal_connect_object (info,
+                                 "peer-vanished",
+                                 G_CALLBACK (on_peer_vanished),
+                                 data->manager, 0);
+
+client_created:
+        if (data->reuse_client)
+                gclue_dbus_manager_complete_get_client (data->manager,
+                                                        data->invocation,
+                                                        path);
+        else
+                gclue_dbus_manager_complete_create_client (data->manager,
+                                                           data->invocation,
+                                                           path);
         goto out;
 
 error_out:
@@ -180,14 +289,36 @@ error_out:
                                                        G_DBUS_ERROR_FAILED,
                                                        error->message);
 out:
-        g_clear_error (&error);
         g_clear_object (&info);
-        g_slice_free (OnClientInfoNewReadyData, data);
-        g_free (path);
+        on_client_info_new_ready_data_free (data);
 
         return FALSE;
 }
 
+static void
+on_peer_vanished_before_completion (GClueClientInfo *info,
+                                    gpointer         user_data)
+{
+        OnClientInfoNewReadyData *data = (OnClientInfoNewReadyData *) user_data;
+        GClueServiceManager *self = GCLUE_SERVICE_MANAGER (data->manager);
+        GClueServiceManagerPrivate *priv = self->priv;
+        const char *bus_name;
+
+        bus_name = gclue_client_info_get_bus_name (info);
+        g_debug ("Client `%s` vanished before agent appeared",
+                 bus_name);
+        g_signal_handlers_disconnect_by_data (info,
+                                              user_data);
+        g_dbus_method_invocation_return_error (data->invocation,
+                                               G_DBUS_ERROR,
+                                               G_DBUS_ERROR_DISCONNECTED,
+                                               "%s vanished before completion",
+                                               bus_name);
+        g_queue_remove (priv->clients_waiting_agent, data);
+        on_client_info_new_ready_data_free (data);
+        g_object_unref (info);
+}
+
 static void
 on_client_info_new_ready (GObject      *source_object,
                           GAsyncResult *res,
@@ -197,9 +328,8 @@ on_client_info_new_ready (GObject      *source_object,
         GClueServiceManagerPrivate *priv = GCLUE_SERVICE_MANAGER (data->manager)->priv;
         GClueClientInfo *info = NULL;
         GClueAgent *agent_proxy;
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
         guint32 user_id;
-        gint64 now;
 
         info = gclue_client_info_new_finish (res, &error);
         if (info == NULL) {
@@ -207,8 +337,7 @@ on_client_info_new_ready (GObject      *source_object,
                                                                G_DBUS_ERROR,
                                                                G_DBUS_ERROR_FAILED,
                                                                error->message);
-                g_error_free (error);
-                g_slice_free (OnClientInfoNewReadyData, data);
+                on_client_info_new_ready_data_free (data);
 
                 return;
         }
@@ -218,52 +347,109 @@ on_client_info_new_ready (GObject      *source_object,
         user_id = gclue_client_info_get_user_id (info);
         agent_proxy = g_hash_table_lookup (priv->agents,
                                            GINT_TO_POINTER (user_id));
-        now = g_get_monotonic_time ();
-        if (agent_proxy == NULL &&
-            now < (priv->init_time + AGENT_WAIT_TIMEOUT_USEC)) {
+        if (agent_proxy == NULL) {
                 /* Its possible that geoclue was just launched on GetClient
                  * call, in which case agents need some time to register
                  * themselves to us.
                  */
-                g_timeout_add (AGENT_WAIT_TIMEOUT,
-                               (GSourceFunc) complete_get_client,
-                               user_data);
+                g_queue_push_tail (priv->clients_waiting_agent, data);
+                g_signal_connect
+                        (info,
+                         "peer-vanished",
+                         G_CALLBACK (on_peer_vanished_before_completion),
+                         user_data);
+                g_debug ("Client `%s` waiting for agent for user ID '%u'",
+                         gclue_client_info_get_bus_name (info), user_id);
+
                 return;
         }
 
         complete_get_client (data);
 }
 
-static gboolean
-gclue_service_manager_handle_get_client (GClueDBusManager      *manager,
-                                         GDBusMethodInvocation *invocation)
+static void
+handle_get_client (GClueDBusManager      *manager,
+                   GDBusMethodInvocation *invocation,
+                   gboolean               reuse_client)
 {
         GClueServiceManager *self = GCLUE_SERVICE_MANAGER (manager);
         GClueServiceManagerPrivate *priv = self->priv;
-        GClueServiceClient *client;
         const char *peer;
         OnClientInfoNewReadyData *data;
 
         peer = g_dbus_method_invocation_get_sender (invocation);
-        client = g_hash_table_lookup (priv->clients, peer);
-        if (client != NULL) {
-                const gchar *existing_path;
-
-                existing_path = gclue_service_client_get_path (client);
-                gclue_dbus_manager_complete_get_client (manager,
-                                                        invocation,
-                                                        existing_path);
-                return TRUE;
-        }
 
         data = g_slice_new (OnClientInfoNewReadyData);
         data->manager = manager;
         data->invocation = invocation;
+        data->reuse_client = reuse_client;
         gclue_client_info_new_async (peer,
                                      priv->connection,
                                      NULL,
                                      on_client_info_new_ready,
                                      data);
+}
+
+static gboolean
+gclue_service_manager_handle_get_client (GClueDBusManager      *manager,
+                                         GDBusMethodInvocation *invocation)
+{
+        handle_get_client (manager, invocation, TRUE);
+
+        return TRUE;
+}
+
+static gboolean
+gclue_service_manager_handle_create_client (GClueDBusManager      *manager,
+                                            GDBusMethodInvocation *invocation)
+{
+        handle_get_client (manager, invocation, FALSE);
+
+        return TRUE;
+}
+
+typedef struct
+{
+        const char *path;
+        const char *bus_name;
+} CompareClientPathNBusNameData;
+
+static int
+compare_client_path_n_bus_name (gconstpointer a,
+                                gconstpointer b)
+{
+        GClueServiceClient *client = GCLUE_SERVICE_CLIENT (a);
+        CompareClientPathNBusNameData *data =
+                (CompareClientPathNBusNameData *) b;
+        GClueClientInfo *info;
+
+        info = gclue_service_client_get_client_info (client);
+
+        if (g_strcmp0 (data->bus_name,
+                       gclue_client_info_get_bus_name (info)) == 0 &&
+            g_strcmp0 (data->path,
+                       gclue_service_client_get_path (client)) == 0)
+                return 0;
+        else
+                return 1;
+}
+
+static gboolean
+gclue_service_manager_handle_delete_client (GClueDBusManager      *manager,
+                                            GDBusMethodInvocation *invocation,
+                                            const char            *path)
+{
+        CompareClientPathNBusNameData data;
+
+        data.path = path;
+        data.bus_name = g_dbus_method_invocation_get_sender (invocation);
+
+        delete_client (GCLUE_SERVICE_MANAGER (manager),
+                       compare_client_path_n_bus_name,
+                       &data);
+
+        gclue_dbus_manager_complete_delete_client (manager, invocation);
+
         return TRUE;
 }
 
@@ -279,6 +465,7 @@ static void
 add_agent_data_free (AddAgentData *data)
 {
         g_clear_pointer (&data->desktop_id, g_free);
+        g_clear_object (&data->manager);
         g_slice_free (AddAgentData, data);
 }
 
@@ -304,7 +491,8 @@ on_agent_proxy_ready (GObject      *source_object,
         GClueServiceManagerPrivate *priv = GCLUE_SERVICE_MANAGER (data->manager)->priv;
         guint32 user_id;
         GClueAgent *agent;
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
+        GList *l;
 
         agent = gclue_agent_proxy_new_for_bus_finish (res, &error);
         if (agent == NULL)
@@ -314,13 +502,26 @@ on_agent_proxy_ready (GObject      *source_object,
         g_debug ("New agent for user ID '%u'", user_id);
         g_hash_table_replace (priv->agents, GINT_TO_POINTER (user_id), agent);
 
-        g_signal_connect (data->info,
-                          "peer-vanished",
-                          G_CALLBACK (on_agent_vanished),
-                          data->manager);
+        g_signal_connect_object (data->info,
+                                 "peer-vanished",
+                                 G_CALLBACK (on_agent_vanished),
+                                 data->manager, 0);
 
         gclue_dbus_manager_complete_add_agent (data->manager, data->invocation);
 
+        l = priv->clients_waiting_agent->head;
+        while (l != NULL) {
+                GList *next = l->next;
+                OnClientInfoNewReadyData *d =
+                        (OnClientInfoNewReadyData *) l->data;
+
+                if (gclue_client_info_get_user_id(d->client_info) == user_id) {
+                        complete_get_client (d);
+                        g_queue_delete_link (priv->clients_waiting_agent, l);
+                }
+                l = next;
+        }
+
         goto out;
 
 error_out:
@@ -329,7 +530,6 @@ error_out:
                                                        G_DBUS_ERROR_FAILED,
                                                        error->message);
 out:
-        g_clear_error (&error);
         add_agent_data_free (data);
 }
 
@@ -341,7 +541,7 @@ on_agent_info_new_ready (GObject      *source_object,
                          gpointer      user_data)
 {
         AddAgentData *data = (AddAgentData *) user_data;
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
         GClueConfig *config;
         const char *xdg_id;
 
@@ -351,7 +551,6 @@ on_agent_info_new_ready (GObject      *source_object,
                                                                G_DBUS_ERROR,
                                                                G_DBUS_ERROR_FAILED,
                                                                error->message);
-                g_error_free (error);
                 add_agent_data_free (data);
 
                 return;
@@ -395,7 +594,7 @@ gclue_service_manager_handle_add_agent (GClueDBusManager      *manager,
         peer = g_dbus_method_invocation_get_sender (invocation);
 
         data = g_slice_new0 (AddAgentData);
-        data->manager = manager;
+        data->manager = g_object_ref (manager);
         data->invocation = invocation;
         data->desktop_id = g_strdup (id);
         gclue_client_info_new_async (peer,
@@ -413,8 +612,17 @@ gclue_service_manager_finalize (GObject *object)
 
         g_clear_object (&priv->locator);
         g_clear_object (&priv->connection);
-        g_clear_pointer (&priv->clients, g_hash_table_unref);
+        if (priv->clients != NULL) {
+                g_list_free_full (priv->clients, g_object_unref);
+                priv->clients = NULL;
+        }
         g_clear_pointer (&priv->agents, g_hash_table_unref);
+        if (priv->clients_waiting_agent != NULL) {
+                g_queue_free_full (priv->clients_waiting_agent,
+                                   on_client_info_new_ready_data_free);
+                priv->clients_waiting_agent = NULL;
+        }
+        g_clear_handle_id (&priv->unix_signal_source, g_source_remove);
 
         /* Chain up to the parent class */
         G_OBJECT_CLASS (gclue_service_manager_parent_class)->finalize (object);
@@ -434,7 +642,7 @@ gclue_service_manager_get_property (GObject    *object,
                 break;
 
         case PROP_ACTIVE:
-                g_value_set_boolean (value, manager->priv->active);
+                g_value_set_boolean (value, (manager->priv->num_clients != 0));
                 break;
 
         default:
@@ -477,18 +685,20 @@ on_avail_accuracy_level_changed (GObject    *object,
 static void
 gclue_service_manager_constructed (GObject *object)
 {
-        GClueServiceManagerPrivate *priv = GCLUE_SERVICE_MANAGER (object)->priv;
+        GClueServiceManager *manager = GCLUE_SERVICE_MANAGER (object);
+        GClueServiceManagerPrivate *priv = manager->priv;
 
         G_OBJECT_CLASS (gclue_service_manager_parent_class)->constructed (object);
 
         priv->locator = gclue_locator_new (GCLUE_ACCURACY_LEVEL_EXACT);
-        g_signal_connect (G_OBJECT (priv->locator),
-                          "notify::available-accuracy-level",
-                          G_CALLBACK (on_avail_accuracy_level_changed),
-                          object);
+        g_signal_connect_object (G_OBJECT (priv->locator),
+                                 "notify::available-accuracy-level",
+                                 G_CALLBACK (on_avail_accuracy_level_changed),
+                                 object, 0);
         on_avail_accuracy_level_changed (G_OBJECT (priv->locator),
                                          NULL,
                                          object);
+        priv->unix_signal_source = g_unix_signal_add (SIGUSR1, log_client_list, manager);
 }
 
 static void
@@ -502,8 +712,6 @@ gclue_service_manager_class_init (GClueServiceManagerClass *klass)
         object_class->set_property = gclue_service_manager_set_property;
         object_class->constructed = gclue_service_manager_constructed;
 
-        g_type_class_add_private (object_class, sizeof (GClueServiceManagerPrivate));
-
         gParamSpecs[PROP_CONNECTION] = g_param_spec_object ("connection",
                                                             "Connection",
                                                             "DBus Connection",
@@ -517,8 +725,8 @@ gclue_service_manager_class_init (GClueServiceManagerClass *klass)
         /**
          * GClueServiceManager:active:
          *
-         * Unlike the D-Bus 'InUse' property, this doesn't differentiate
-         * between system components and apps.
+         * Unlike the D-Bus 'InUse' property, this indicates wether or not
+         * currently we have connected clients.
          */
         gParamSpecs[PROP_ACTIVE] = g_param_spec_boolean ("active",
                                                          "Active",
@@ -533,19 +741,13 @@ gclue_service_manager_class_init (GClueServiceManagerClass *klass)
 static void
 gclue_service_manager_init (GClueServiceManager *manager)
 {
-        manager->priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
-                                                     GCLUE_TYPE_SERVICE_MANAGER,
-                                                     GClueServiceManagerPrivate);
+        manager->priv = gclue_service_manager_get_instance_private (manager);
 
-        manager->priv->clients = g_hash_table_new_full (g_str_hash,
-                                                        g_str_equal,
-                                                        g_free,
-                                                        g_object_unref);
         manager->priv->agents = g_hash_table_new_full (g_direct_hash,
                                                        g_direct_equal,
                                                        NULL,
                                                        g_object_unref);
-        manager->priv->init_time = g_get_monotonic_time ();
+        manager->priv->clients_waiting_agent = g_queue_new ();
 }
 
 static gboolean
@@ -564,6 +766,10 @@ static void
 gclue_service_manager_manager_iface_init (GClueDBusManagerIface *iface)
 {
         iface->handle_get_client = gclue_service_manager_handle_get_client;
+        iface->handle_create_client =
+                gclue_service_manager_handle_create_client;
+        iface->handle_delete_client =
+                gclue_service_manager_handle_delete_client;
         iface->handle_add_agent = gclue_service_manager_handle_add_agent;
 }
 
@@ -587,5 +793,5 @@ gclue_service_manager_new (GDBusConnection *connection,
 gboolean
 gclue_service_manager_get_active (GClueServiceManager *manager)
 {
-        return manager->priv->active;
+        return (manager->priv->num_clients != 0);
 }
